@@ -18,7 +18,7 @@
 #
 #------------------------------------------------------------------------
 #
-# $Id: Exception.pm,v 2.64 2004/01/13 16:19:10 abw Exp $
+# $Id: Exception.pm,v 2.66 2006/01/30 20:04:49 abw Exp $
 #
 #========================================================================
 
@@ -36,7 +36,7 @@ use constant TEXT  => 2;
 use overload q|""| => "as_string", fallback => 1;
 
 
-$VERSION = sprintf("%d.%02d", q$Revision: 2.64 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.66 $ =~ /(\d+)\.(\d+)/);
 
 
 #------------------------------------------------------------------------
@@ -219,21 +219,21 @@ would be a specific type of the more general 'foo' type.
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@andywardley.comE<gt>
+Andy Wardley E<lt>abw@wardley.orgE<gt>
 
-L<http://www.andywardley.com/|http://www.andywardley.com/>
+L<http://wardley.org/|http://wardley.org/>
 
 
 
 
 =head1 VERSION
 
-2.64, distributed as part of the
-Template Toolkit version 2.13, released on 30 January 2004.
+2.66, distributed as part of the
+Template Toolkit version 2.15, released on 26 May 2006.
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2004 Andy Wardley.  All Rights Reserved.
+  Copyright (C) 1996-2006 Andy Wardley.  All Rights Reserved.
   Copyright (C) 1998-2002 Canon Research Centre Europe Ltd.
 
 This module is free software; you can redistribute it and/or
